part of maps;

/// A Render object widget which draws tooltip shape.
class _MapTooltip extends LeafRenderObjectWidget {
  const _MapTooltip({
    this.tooltipSettings,
    this.mapDelegate,
    this.enableShapeTooltip,
    this.enableBubbleTooltip,
    this.mapsThemeData,
    this.state,
  });

  final MapTooltipSettings tooltipSettings;

  final MapShapeLayerDelegate mapDelegate;

  final bool enableShapeTooltip;

  final bool enableBubbleTooltip;

  final SfMapsThemeData mapsThemeData;

  final _MapsShapeLayerState state;

  @override
  _RenderMapTooltip createRenderObject(BuildContext context) {
    return _RenderMapTooltip(
        tooltipSettings: tooltipSettings,
        mapDelegate: mapDelegate,
        mediaQueryData: MediaQuery.of(context),
        enableShapeTooltip: enableShapeTooltip,
        enableBubbleTooltip: enableBubbleTooltip,
        mapsThemeData: mapsThemeData,
        state: state);
  }

  @override
  void updateRenderObject(
      BuildContext context, _RenderMapTooltip renderObject) {
    renderObject
      ..tooltipSettings = tooltipSettings
      ..mapDelegate = mapDelegate
      ..enableShapeTooltip = enableShapeTooltip
      ..enableBubbleTooltip = enableBubbleTooltip
      ..mapsThemeData = mapsThemeData
      ..mediaQueryData = MediaQuery.of(context);
  }
}

class _RenderMapTooltip extends RenderBox {
  _RenderMapTooltip({
    MapTooltipSettings tooltipSettings,
    MapShapeLayerDelegate mapDelegate,
    bool enableShapeTooltip,
    bool enableBubbleTooltip,
    SfMapsThemeData mapsThemeData,
    MediaQueryData mediaQueryData,
    _MapsShapeLayerState state,
  })  : _tooltipSettings = tooltipSettings,
        _mapDelegate = mapDelegate,
        _enableShapeTooltip = enableShapeTooltip,
        _enableBubbleTooltip = enableBubbleTooltip,
        _mapsThemeData = mapsThemeData,
        _mediaQueryData = mediaQueryData,
        _state = state {
    _textPainter = TextPainter(textDirection: TextDirection.ltr);
    _textPainter.textScaleFactor = _mediaQueryData.textScaleFactor;
    _tooltipAnimation = CurvedAnimation(
        parent: _state.tooltipAnimationController, curve: Curves.easeOutBack);
    _mouseTrackerAnnotation = MouseTrackerAnnotation(
      onHover: _onHover,
    );
  }

  final _MapsShapeLayerState _state;

  final _TooltipShape _tooltipShape = const _TooltipShape();

  Animation<double> _tooltipAnimation;

  TextPainter _textPainter;

  MouseTrackerAnnotation _mouseTrackerAnnotation;

  String _tooltipText;

  Offset _touchPosition;

  MapTooltipSettings get tooltipSettings => _tooltipSettings;
  MapTooltipSettings _tooltipSettings;
  set tooltipSettings(MapTooltipSettings value) {
    if (_tooltipSettings == value) {
      return;
    }
    _tooltipSettings = value;
  }

  MapShapeLayerDelegate get mapDelegate => _mapDelegate;
  MapShapeLayerDelegate _mapDelegate;
  set mapDelegate(MapShapeLayerDelegate value) {
    if (_mapDelegate == value) {
      return;
    }
    _mapDelegate = value;
    _tooltipText = null;
    // Tooltip will remain in the UI for 3 seconds.
    // Between this time when mapDelegate changes,
    // We need to remove the tooltip from the UI immediately.
    if (_state.tooltipAnimationController.value > 0) {
      _state.tooltipAnimationController.reverse();
    }
  }

  bool get enableShapeTooltip => _enableShapeTooltip;
  bool _enableShapeTooltip;
  set enableShapeTooltip(bool value) {
    if (_enableShapeTooltip == value) {
      return;
    }
    _enableShapeTooltip = value;
  }

  bool get enableBubbleTooltip => _enableBubbleTooltip;
  bool _enableBubbleTooltip;
  set enableBubbleTooltip(bool value) {
    if (_enableBubbleTooltip == value) {
      return;
    }
    _enableBubbleTooltip = value;
  }

  SfMapsThemeData get mapsThemeData => _mapsThemeData;
  SfMapsThemeData _mapsThemeData;
  set mapsThemeData(SfMapsThemeData value) {
    if (_mapsThemeData == value) {
      return;
    }
    _mapsThemeData = value;
    markNeedsPaint();
  }

  MediaQueryData get mediaQueryData => _mediaQueryData;
  MediaQueryData _mediaQueryData;
  set mediaQueryData(MediaQueryData value) {
    if (_mediaQueryData == value) {
      return;
    }
    _mediaQueryData = value;
    _textPainter.textScaleFactor = _mediaQueryData.textScaleFactor;
    markNeedsPaint();
  }

  void _onHover(PointerHoverEvent event) {
    final _MapChildParentData childParentData = parentData;
    final Offset currentPosition =
        event.localPosition - childParentData.parentOffset;
    double bubbleRadius;
    String tooltipText;
    final int length = childParentData.mapDataSource.values.length;
    for (int i = 0; i < length; i++) {
      final _MapModel model = childParentData.mapDataSource.values.elementAt(i);
      if (_enableBubbleTooltip &&
          model.bubblePath != null &&
          model.bubblePath.contains(currentPosition)) {
        if (bubbleRadius == null) {
          bubbleRadius = model.bubbleRadius;
          if (_mapDelegate.bubbleTooltipTextMapper != null &&
              model.dataIndex != null) {
            tooltipText = _mapDelegate.bubbleTooltipTextMapper(model.dataIndex);
          } else {
            tooltipText = model.primaryKey;
          }
          _touchPosition = currentPosition;
        } else if (model.bubbleRadius < bubbleRadius) {
          if (_mapDelegate.bubbleTooltipTextMapper != null &&
              model.dataIndex != null) {
            tooltipText = _mapDelegate.bubbleTooltipTextMapper(model.dataIndex);
          } else {
            tooltipText = model.primaryKey;
          }
          _touchPosition = currentPosition;
        }
      } else if (_enableShapeTooltip &&
          model.shapePath != null &&
          model.shapePath.contains(currentPosition)) {
        if (_mapDelegate.shapeTooltipTextMapper != null &&
            model.dataIndex != null) {
          tooltipText = _mapDelegate.shapeTooltipTextMapper(model.dataIndex);
        } else {
          tooltipText = model.primaryKey;
        }
        _touchPosition = currentPosition;
        break;
      }
    }

    if (tooltipText != null) {
      _tooltipText = tooltipText;
      if (_state.tooltipAnimationController.value == 0) {
        _state.tooltipAnimationController.forward(from: 0.0);
      }
    } else {
      _state.tooltipAnimationController.reverse();
      if (_state.tooltipAnimationController.status ==
          AnimationStatus.dismissed) {
        _tooltipText = null;
        _touchPosition = null;
      }
    }
    markNeedsPaint();
  }

  @override
  bool get isRepaintBoundary => true;

  @override
  void performLayout() {
    size = _getActualSize(constraints);
  }

  @override
  void attach(PipelineOwner owner) {
    super.attach(owner);
    _tooltipAnimation.addListener(markNeedsPaint);
  }

  @override
  void detach() {
    _tooltipAnimation.removeListener(markNeedsPaint);
    super.detach();
  }

  @override
  void paint(PaintingContext context, Offset offset) {
    _drawTooltipShape(context, offset);
  }

  void _drawTooltipShape(PaintingContext context, Offset offset) {
    final bool mouseIsConnected =
        RendererBinding.instance.mouseTracker.mouseIsConnected;

    if (mouseIsConnected) {
      final AnnotatedRegionLayer<MouseTrackerAnnotation> layer =
          AnnotatedRegionLayer<MouseTrackerAnnotation>(
        _mouseTrackerAnnotation,
        offset: offset,
      );
      context.pushLayer(layer, super.paint, offset);
    }

    if (_tooltipText != null) {
      _textPainter.text = TextSpan(
          text: _tooltipText,
          style: _tooltipSettings.textStyle ?? _mapsThemeData.tooltipTextStyle);
      _textPainter.layout();

      _tooltipShape.paint(
          context,
          _touchPosition,
          _textPainter,
          Paint()
            ..style = PaintingStyle.fill
            ..color = _tooltipSettings.color ?? _mapsThemeData.tooltipColor,
          this,
          _tooltipAnimation,
          _mapsThemeData,
          _tooltipSettings);
    }
  }

  void _forwardTooltipAnimation() {
    _state.tooltipAnimationController.forward(from: 0.0);
    _state.tooltipDelayTimer?.cancel();
    _state.tooltipDelayTimer = Timer(
      const Duration(seconds: 3),
      () {
        _state.tooltipDelayTimer = null;
        if (_state.tooltipAnimationController.status ==
            AnimationStatus.completed) {
          _state.tooltipAnimationController.reverse();
          if (_state.tooltipAnimationController.status ==
              AnimationStatus.dismissed) {
            _tooltipText = null;
            _touchPosition = null;
          }
        }
      },
    );
  }
}
