part of maps;

enum _MarkerAction { insert, removeAt, replace, clear, none }

class MapShapeLayerController extends ChangeNotifier {
  List<int> _replaceableIndices;

  int _markersCount;

  int _index = -1;

  _MarkerAction _markerAction = _MarkerAction.none;

  int get markersCount => _markersCount;

  int get selectedIndex => _selectedIndex;
  int _selectedIndex;
  set selectedIndex(int value) {
    if (_selectedIndex == value) {
      return;
    }
    _selectedIndex = value;
    notifyListeners();
  }

  /// Adds marker dynamically.
  ///
  /// If the [MapShapeLayer.initialMarkersCount] is 10 and if the index given for
  /// the insertMarker method is 10 which is greater than the available indices,
  /// then the marker will be added as a last item.
  ///
  /// See also:
  /// * [MapShapeLayer.markerBuilder], to return the [MapMarker].
  void insertMarker(int index) {
    _markerAction = _MarkerAction.insert;
    assert(index <= markersCount);
    if (index > markersCount) {
      index = markersCount;
    }
    _index = index;
    notifyListeners();
  }

  void removeMarkerAt(int index) {
    _markerAction = _MarkerAction.removeAt;
    _index = index;
    notifyListeners();
  }

  /// Updates the markers in the given indices dynamically.
  /// See also:
  /// * [MapShapeLayer.markerBuilder], to return the updated [MapMarker].
  void updateMarkers(List<int> indices) {
    _markerAction = _MarkerAction.replace;
    _replaceableIndices = indices;
    notifyListeners();
  }

  void clearMarkers() {
    _markerAction = _MarkerAction.clear;
    notifyListeners();
  }

  @override
  void dispose() {
    _replaceableIndices?.clear();
    _replaceableIndices = null;

    super.dispose();
  }
}
